@@ -1,6 +1,6 @@
 /*
 Weborf
-Copyright (C) 2017-2018  Salvo "LtWorf" Tomaselli
+Copyright (C) 2017-2020  Salvo "LtWorf" Tomaselli
 
 Weborf is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -87,7 +87,7 @@ static inline int check_etag(connection_t* connection_prop,char *a) {
         time_t etag=(time_t)strtol(a+1,NULL,0);
         if (connection_prop->strfile_stat.st_mtime==etag) {
             //Browser has the item in its cache, sending 304
-            send_http_header(304,0,NULL,true,etag,connection_prop);
+            send_http_header(304, NULL, NULL, true, etag, connection_prop);
             return 0;
         }
     }
@@ -492,7 +492,7 @@ static inline int options (connection_t* connection_prop) {
 #define ALLOWED "Allow: GET,POST,PUT,DELETE,OPTIONS\r\n"
 #endif
 
-    send_http_header(200,0,ALLOWED,true,-1,connection_prop);
+    send_http_header(200, NULL, ALLOWED, true, -1, connection_prop);
     return 0;
 }
 
@@ -582,6 +582,7 @@ escape:
  * the appropriate index file or show the list of the files.
  * */
 static int get_or_post(connection_t *connection_prop, string_t post_param) {
+    unsigned long long int size_zero = 0;
 
     if (S_ISDIR(connection_prop->strfile_stat.st_mode)) {//Requested a directory
 
@@ -593,7 +594,7 @@ static int get_or_post(connection_t *connection_prop, string_t post_param) {
         if (!endsWith(connection_prop->strfile,"/",connection_prop->strfile_len,1)) {//Putting the ending / and redirect
             char head[URI_LEN+12];//12 is the size for the location header
             snprintf(head,URI_LEN+12,"Location: %s/\r\n",connection_prop->page);
-            send_http_header(301,0,head,true,-1,connection_prop);
+            send_http_header(301, &size_zero, head, true, -1, connection_prop);
             return 0;
         } else {//Requested directory with "/" Search for index files or list directory
 
@@ -606,7 +607,7 @@ static int get_or_post(connection_t *connection_prop, string_t post_param) {
                 if (file_exists(connection_prop->strfile)) { //If index exists, redirect to it
                     char head[URI_LEN+12];//12 is the size for the location header
                     snprintf(head,URI_LEN+12,"Location: %s%s\r\n",connection_prop->page,weborf_conf.indexes[i]);
-                    send_http_header(303,0,head,true,-1,connection_prop);
+                    send_http_header(303, &size_zero, head, true, -1, connection_prop);
                     return 0;
                 }
             }
@@ -647,8 +648,11 @@ static int send_error_header(int retval, connection_t *connection_prop) {
         return send_err(connection_prop,500,"Internal server error");
     case ERR_FILENOTFOUND:
         return send_err(connection_prop,404,"Page not found");
+    case ERR_SERVICE_UNAVAILABLE:
     case ERR_NOMEM:
         return send_err(connection_prop,503,"Service Unavailable");
+    case ERR_RANGE_NOT_SATISFIABLE:
+        return send_err(connection_prop,416,"Range not satisfiable");
     case ERR_NODATA:
     case ERR_NOTHTTP:
         return send_err(connection_prop,400,"Bad request");
@@ -656,8 +660,6 @@ static int send_error_header(int retval, connection_t *connection_prop) {
         return send_err(connection_prop,403,"Forbidden");
     case ERR_NOTIMPLEMENTED:
         return send_err(connection_prop,501,"Not implemented");
-    case ERR_SERVICE_UNAVAILABLE:
-        return send_err(connection_prop,503,"Service Unavailable");
     case ERR_PRECONDITION_FAILED:
         return send_err(connection_prop,412,"Precondition Failed");
     case ERR_CONFLICT:
@@ -669,9 +671,9 @@ static int send_error_header(int retval, connection_t *connection_prop) {
     case ERR_NOAUTH:
         return request_auth(connection_prop);//Sends a request for authentication
     case OK_NOCONTENT:
-        return send_http_header(204,0,NULL,true,-1,connection_prop);
+        return send_http_header(204, NULL, NULL, true, -1, connection_prop);
     case OK_CREATED:
-        return send_http_header(201,0,NULL,true,-1,connection_prop);
+        return send_http_header(201, NULL, NULL, true, -1, connection_prop);
     }
     return 0; //Make gcc happy
 }
@@ -700,7 +702,7 @@ int write_dir(char* real_basedir,connection_t* connection_prop) {
     if (cache_send_item(0,connection_prop)) return 0;
 
 
-    int pagelen;
+    unsigned long long int pagelen;
     bool parent;
 
     /*
@@ -749,7 +751,7 @@ int write_dir(char* real_basedir,connection_t* connection_prop) {
         */
         send_http_header(
             200,
-            pagelen,
+            &pagelen,
             "Content-Type: text/html\r\n",
             true,
             connection_prop->strfile_stat.st_mtime,
@@ -803,7 +805,7 @@ static inline int write_compressed_file(connection_t* connection_prop ) {
 
     connection_prop->keep_alive=false;
     send_http_header(200,
-                     connection_prop->strfile_stat.st_size,
+                     &connection_prop->strfile_stat.st_size,
                      "Content-Encoding: gzip\r\n",
                      false,
                      connection_prop->strfile_stat.st_mtime,
@@ -837,7 +839,7 @@ static inline int write_compressed_file(connection_t* connection_prop ) {
  *
  * a must be a pointer to a buffer large at least RBUFFER+MIMETYPELEN+16
  * */
-static inline unsigned long long int bytes_to_send(connection_t* connection_prop,char *a) {
+static inline unsigned long long int bytes_to_send(connection_t* connection_prop, char *a, int *errcode) {
     int http_code=200;
     errno=0;
     unsigned long long int count;
@@ -864,8 +866,8 @@ static inline unsigned long long int bytes_to_send(connection_t* connection_prop
      * If range header is present and (If-Range has the same etag OR there is no If-Range)
      * */
     if (range_header && connection_prop->strfile_stat.st_mtime==etag) {//Find if it is a range request 5 is strlen of "range"
-
-        unsigned long long int from,to;
+        unsigned long long int from;
+        unsigned long long int to;
 
         {
             //Locating from and to
@@ -873,35 +875,41 @@ static inline unsigned long long int bytes_to_send(connection_t* connection_prop
             char *eq, *sep;
 
             if ((eq=strstr(a,"="))==NULL ||(sep=strstr(eq,"-"))==NULL) {//Invalid data in Range header.
-                errno =ERR_NOTHTTP;
-                return ERR_NOTHTTP;
+                *errcode = ERR_NOTHTTP;
+                return 0;
             }
             sep[0]=0;
             from=strtoll(eq+1,NULL,0);
             to=strtoll(sep+1,NULL,0);
-        }
 
-        if (to==0) { //If no to is specified, it is to the end of the file
-            to=connection_prop->strfile_stat.st_size-1;
+            if (to == 0 && strlen(sep +1) == 0) { //If no to is specified, it is to the end of the file
+                to=connection_prop->strfile_stat.st_size-1;
+            }
         }
 
         http_code=206;
 
-        t=snprintf(hbuf,remain,"Accept-Ranges: bytes\r\nContent-Range: bytes %llu-%llu/%lld\r\n",(unsigned long long int)from,(unsigned long long int)to,(long long int)connection_prop->strfile_stat.st_size);
+        t=snprintf(hbuf,remain,"Accept-Ranges: bytes\r\nContent-Range: bytes %llu-%llu/%lld\r\n", from, to,(long long int)connection_prop->strfile_stat.st_size);
         hbuf+=t;
         remain-=t;
 
-        lseek(connection_prop->strfile_fd,from,SEEK_SET);
-        count=to-from+1;
+        count = to - from + 1;
+        if (from >= connection_prop->strfile_stat.st_size || from + count > connection_prop->strfile_stat.st_size) {
+            *errcode = ERR_RANGE_NOT_SATISFIABLE;
+            return 0;
+        }
+
+        if (from)
+            lseek(connection_prop->strfile_fd,from,SEEK_SET);
+
 
     } else //Normal request
 #endif
     {
         a[0]=0; //Reset buffer in case it isn't used for headers
-        count=connection_prop->strfile_stat.st_size;
+        count = connection_prop->strfile_stat.st_size;
     }
 
-
     //Sending MIME to the client
 #ifdef SEND_MIMETYPES
     if (weborf_conf.send_content_type) {
@@ -914,8 +922,7 @@ static inline unsigned long long int bytes_to_send(connection_t* connection_prop
         //remain-=t;
     }
 #endif
-
-    send_http_header(http_code,count,a,true,connection_prop->strfile_stat.st_mtime,connection_prop);
+    send_http_header(http_code, &count,a,true,connection_prop->strfile_stat.st_mtime,connection_prop);
     return count;
 }
 
@@ -954,7 +961,10 @@ int write_file(connection_t* connection_prop) {
 
     //Determines how many bytes send, depending on file size and ranges
     //Also sends the http header
-    unsigned long long int count= bytes_to_send(connection_prop,&a[0]);
+    int errcode = 0;
+    unsigned long long int count = bytes_to_send(connection_prop,&a[0], &errcode);
+    if (errcode != 0)
+        return errcode;
     /*if (errno !=0) {
         int e=errno;
         errno=0;
@@ -1138,7 +1148,7 @@ This function will automatically take care of generating Connection header when
 needed, according to keep_alive and protocol_version of connection_prop
 
 */
-int send_http_header(int code, unsigned long long int size,char* headers,bool content,time_t timestamp,connection_t* connection_prop) {
+int send_http_header(int code, unsigned long long int *size,char* headers,bool content,time_t timestamp,connection_t* connection_prop) {
     fd_t sock = connection_prop->sock;
     int len_head,wrote;
     char *head=malloc(HEADBUF);
@@ -1198,12 +1208,12 @@ int send_http_header(int code, unsigned long long int size,char* headers,bool co
     }
 #endif
 
-    if (size>0 || (connection_prop->keep_alive==true)) {
+    if (size != NULL && connection_prop->keep_alive==true) {
         //Content length (or entity length) and extra headers
         if (content) {
-            len_head=snprintf(head,left_head,"Content-Length: %llu\r\n",(unsigned long long int)size);
+            len_head=snprintf(head,left_head,"Content-Length: %llu\r\n", *size);
         } else {
-            len_head=snprintf(head,left_head,"entity-length: %llu\r\n",(unsigned long long int)size);
+            len_head=snprintf(head,left_head,"entity-length: %llu\r\n", *size);
         }
 
         head+=len_head;
@@ -1244,7 +1254,7 @@ static int tar_send_dir(connection_t* connection_prop) {
             );
 
     send_http_header(200,
-                     0,
+                     NULL,
                      headers,
                      true,
                      -1,
