@@ -1,13 +1,13 @@
 from distutils.core import setup
 
 setup(
-    version='0.16',
+    version='0.17',
     name='qweborf',
     author="Salvo 'LtWorf' Tomaselli",
     author_email='tiposchi@tiscali.it',
     maintainer="Salvo 'LtWorf' Tomaselli",
     maintainer_email='tiposchi@tiscali.it',
-    url='http://ltworf.github.io/weborf/',
+    url='https://ltworf.github.io/weborf/',
     license='GPL3',
     packages=('qweborf',),
     )
