@@ -1,6 +1,6 @@
 /*
 Weborf
-Copyright (C) 2009-2018  Salvo "LtWorf" Tomaselli
+Copyright (C) 2009-2020  Salvo "LtWorf" Tomaselli
 
 Weborf is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -241,13 +241,13 @@ static inline int printprops(fd_t sock, char *page, u_dav_details props,char* fi
     pagesize += printf_s;
 
     if (props.dav_details.getetag) {
-        printf_s = snprintf(xml + pagesize, maxsize, "<D:getetag>%lld</D:getetag>\n", stat_s.st_mtime);
+        printf_s = snprintf(xml + pagesize, maxsize, "<D:getetag>%lld</D:getetag>\n", (long long int)stat_s.st_mtime);
         maxsize -= printf_s;
         pagesize += printf_s;
     }
 
     if (props.dav_details.getcontentlength) {
-        printf_s = snprintf(xml + pagesize, maxsize, "<D:getcontentlength>%lld</D:getcontentlength>\n", stat_s.st_size);
+        printf_s = snprintf(xml + pagesize, maxsize, "<D:getcontentlength>%lld</D:getcontentlength>\n", (long long int)stat_s.st_size);
         maxsize -= printf_s;
         pagesize += printf_s;
     }
@@ -289,7 +289,7 @@ static inline int printprops(fd_t sock, char *page, u_dav_details props,char* fi
     printf_s = snprintf(xml + pagesize, maxsize,
         "</D:prop><D:status>HTTP/1.1 200 OK</D:status></D:propstat>"
         "</D:response>");
-    maxsize -= printf_s;
+//     maxsize -= printf_s;
     pagesize += printf_s;
 
     myio_write(sock, xml, pagesize);
@@ -321,7 +321,7 @@ int propfind(connection_t* connection_prop, string_t *post_param) {
         if (S_ISDIR(connection_prop->strfile_stat.st_mode) && !endsWith(connection_prop->strfile,"/",connection_prop->strfile_len,1)) {//Putting the ending / and redirect
             char head[URI_LEN+12];//12 is the size for the location header
             snprintf(head,URI_LEN+12,"Location: %s/\r\n",connection_prop->page);
-            send_http_header(301,0,head,true,-1,connection_prop);
+            send_http_header(301, NULL, head, true, -1, connection_prop);
             return 0;
         }
     } // End redirection
@@ -339,7 +339,7 @@ int propfind(connection_t* connection_prop, string_t *post_param) {
 
     //Sets keep alive to false (have no clue about how big is the generated xml) and sends a multistatus header code
     connection_prop->keep_alive=false;
-    send_http_header(207,0,"Content-Type: text/xml; charset=\"utf-8\"\r\n",false,-1,connection_prop);
+    send_http_header(207, NULL, "Content-Type: text/xml; charset=\"utf-8\"\r\n", false, -1, connection_prop);
 
     //Check if exists in cache
     if (has_cache) {
@@ -481,6 +481,11 @@ int copy_move(connection_t* connection_prop) {
         retval=ERR_NOTHTTP;
         goto escape;
     }
+    // Overwrite can only be "T" or "F"
+    if (overwrite_b && (!(overwrite[0] == 'T' || overwrite[0] == 'F') || overwrite[1] != '\0')) {
+        retval = ERR_NOTHTTP;
+        goto escape;
+    }
 
     /*Sets if there is overwrite or not.
     ovewrite header is a boolean where F is false.
@@ -489,12 +494,15 @@ int copy_move(connection_t* connection_prop) {
         check_exists=(overwrite[0]!='F');
     }
 
-    dest=strstr(dest,host);
-    if (dest==NULL) {//Something is wrong here
-        retval=ERR_NOTHTTP;
-        goto escape;
+    //Destination might be like http://localhost/destination or just /destination
+    if (dest[0] != '/') {
+        dest=strstr(dest, host);
+        if (dest == NULL) {//Something is wrong here
+            retval = ERR_NOTHTTP;
+            goto escape;
+        }
+        dest += strlen(host);
     }
-    dest+=strlen(host);
 
     //Local path for destination file
     snprintf(destination,PATH_LEN,"%s%s",connection_prop->basedir,dest);
