@@ -21,19 +21,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef WEBORF_WEBDAV_H
 #define WEBORF_WEBDAV_H
 
-#include <string.h>
-#include <stdbool.h>
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <time.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
+#include "types.h"
 
-#include "instance.h"
-
-int propfind(int sock,connection_t* connection_prop,string_t *post_param);
-int mkcol(int sock,connection_t* connection_prop);
-int copy_move(int sock,connection_t* connection_prop);
+int propfind(connection_t* connection_prop,string_t *post_param);
+int mkcol(connection_t* connection_prop);
+int copy_move(connection_t* connection_prop);
 #endif
